
;******************************* Sub-Debug *********************************
; Sub: Sendet Text ber die serielle Schnittstelle
; Parameter:   Z enthlt den Zeiger auf den Text im ROM
; Return:
; Scratch-Reg: COUNTER_1, COPY, TEMP_0, ZL, ZH
;***************************************************************************
send_text:
        lpm		TEMP_0,Z+
		tst		TEMP_0
		breq	send_text_end			; Ende

        rcall   send_byte

		brne    send_text				; weiter

send_text_end:
        ret

;******************************* Sub-Debug *********************************
; Sub: Sendet NL/CR ber die serielle Schnittstelle
; Parameter:   -
; Return:      -
; Scratch-Reg: TEMP_0
;***************************************************************************
send_newline:
        push    TEMP_0
		ldi     TEMP_0,$0A
        rcall   send_byte
		ldi     TEMP_0,$0D
        rcall   send_byte
        pop     TEMP_0
		ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 als HEX-Dump ber UART
; Parameter:
; Return:
; Scratch-Reg: TEMP_0, TEMP_1
;***************************************************************************
send_byte_hex:
        push TEMP_0
		push TEMP_1
        ldi  TEMP_1,7
        push TEMP_0
		swap TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30                    ; nach Ziffer
		cpi  TEMP_0,$3A                    ; 0...9
    	brlo wb_1
        add  TEMP_0,TEMP_1                 ; A...F
wb_1:
        rcall send_byte
		pop  TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30                   ; nach Ziffer
		cpi  TEMP_0,$3A                   ; 0...9
		brlo wb_2
        add  TEMP_0,TEMP_1                ; A...F
wb_2:
        rcall send_byte
        pop   TEMP_1
		pop   TEMP_0
        ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 als Dezimalwert ber UART
; Parameter: TEMP_0
; Return:	 TEMP_2, TEMP_1, TEMP_0
; Scratch-Reg: TEMP_0, TEMP_1
;***************************************************************************
send_byte_dez:
		clr		TEMP_1
		clr		TEMP_2

loop_100:
		inc		TEMP_2
		subi	TEMP_0,100
		brcc	loop_100
		
		dec		TEMP_2				; letztes wieder zurck
		subi	TEMP_0,-100							

loop_10:
		inc		TEMP_1
		subi	TEMP_0,10
		brcc	loop_10
		
		dec		TEMP_1				; letztes wieder zurck
		subi	TEMP_0,-10							

		ori		TEMP_2,$30		
		ori		TEMP_1,$30
		ori		TEMP_0,$30
		push	TEMP_0
		mov		TEMP_0,TEMP_2
		rcall	send_byte
		mov		TEMP_0,TEMP_1
		rcall	send_byte
		pop		TEMP_0
		rcall	send_byte
			
        ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 ber UART
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************
send_byte:
        sbis    UCSRA,UDRE                  ; Send-Register leer ?
        rjmp    send_byte              	    ; nein

        out     UDR,TEMP_0
        ret

;******************************** Sub-Intern *******************************
; Sub: Speicher als 32-Bit (4-Byte) Tabelle ausgeben
; Parameter:    Z - Adresse
; Parameter:    Y - Lnge
; Return:       -
; Scratch-Reg:  -
;***************************************************************************
debug_32bit_tab:
        rcall    send_text
        rcall    send_newline

        lsr        YH                    ; /4
        ror        YL
        lsr        YH
        ror        YL

debug_32bit_tab_line_loop:
        mov        TEMP_0,ZH            ; Adresse senden
        rcall    send_byte_hex
        mov        TEMP_0,ZL
        rcall    send_byte_hex

        ldi        TEMP_1,4            ; Bytecounter

debug_32bit_tab_byte_loop:
        ldi        TEMP_0,' '            ; Leerzeichen
        rcall    send_byte

        ld        TEMP_0,X+
        rcall    send_byte_hex

        dec        TEMP_1
        brne    debug_32bit_tab_byte_loop

        rcall    send_newline
        sbiw    Y,1
        brne    debug_32bit_tab_line_loop

        ret
